@@ -1,4 +1,3 @@
-from waffle_utils.file import io
 from waffle_utils.utils import type_validator
 
 from . import BaseField
@@ -8,7 +7,7 @@ class Category(BaseField):
     def __init__(
         self,
         # required
-        cat_id: int,
+        category_id: int,
         supercategory: str,
         name: str,
         # for keypoint detection
@@ -16,7 +15,7 @@ class Category(BaseField):
         skeleton: list[list[int]] = None,
     ):
 
-        self.cat_id = cat_id
+        self.category_id = category_id
         self.supercategory = supercategory
         self.name = name
         self.keypoints = keypoints
@@ -24,15 +23,17 @@ class Category(BaseField):
 
     # properties
     @property
-    def cat_id(self):
-        return self.__cat_id
+    def category_id(self):
+        return self.__category_id
 
-    @cat_id.setter
+    @category_id.setter
     @type_validator(int)
-    def cat_id(self, v):
+    def category_id(self, v):
+        if v is None:
+            raise ValueError("category_id should not be None")
         if v and v < 1:
             raise ValueError("id should be greater than 0.")
-        self.__cat_id = v
+        self.__category_id = v
 
     @property
     def supercategory(self):
@@ -74,7 +75,7 @@ class Category(BaseField):
     @classmethod
     def new(
         cls,
-        cat_id: int,
+        category_id: int,
         supercategory: str,
         name: str,
         keypoints: list[str] = None,
@@ -83,7 +84,7 @@ class Category(BaseField):
         """Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
             keypoints (list[str]): category name.
@@ -92,98 +93,60 @@ class Category(BaseField):
         Returns:
             Category: category class
         """
-        return cls(cat_id, supercategory, name, keypoints, skeleton)
-
-    @classmethod
-    def from_dict(cls, d: dict) -> "Category":
-        """Category Format from dictionary
-
-        Args:
-            d (dict): Category dictionary
-
-        Returns:
-            Category: category class
-        """
-        cat_id = d.get("id", None)
-        supercategory = d.get("supercategory", None)
-        name = d.get("name", None)
-        keypoints = d.get("keypoints", None)
-        skeleton = d.get("skeleton", None)
-
-        if cat_id is None:
-            raise ValueError("id field missing")
-        if supercategory is None:
-            raise ValueError("file_name field missing")
-        if name is None:
-            raise ValueError("width field missing")
-
-        return cls(cat_id, supercategory, name, keypoints, skeleton)
-
-    @classmethod
-    def from_json(cls, f: str) -> "Category":
-        """Category Format from json file
-
-        Args:
-            d (dict): Category json file
-
-        Returns:
-            Category: Category class
-        """
-        d: dict = io.load_json(f)
-        return cls.from_dict(d)
+        return cls(category_id, supercategory, name, keypoints, skeleton)
 
     @classmethod
     def classification(
-        cls, cat_id: int, supercategory: str, name: str
+        cls, category_id: int, supercategory: str, name: str
     ) -> "Category":
         """Classification Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
 
         Returns:
             Category: category class
         """
-        return cls(cat_id, supercategory, name)
+        return cls(category_id, supercategory, name)
 
     @classmethod
     def object_detection(
-        cls, cat_id: int, supercategory: str, name: str
+        cls, category_id: int, supercategory: str, name: str
     ) -> "Category":
         """Object Detection Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
 
         Returns:
             Category: category class
         """
-        return cls(cat_id, supercategory, name)
+        return cls(category_id, supercategory, name)
 
     @classmethod
     def segmentation(
-        cls, cat_id: int, supercategory: str, name: str
+        cls, category_id: int, supercategory: str, name: str
     ) -> "Category":
         """Segmentation Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
 
         Returns:
             Category: category class
         """
-        return cls(cat_id, supercategory, name)
+        return cls(category_id, supercategory, name)
 
     @classmethod
     def keypoint_detection(
         cls,
-        cat_id: int,
+        category_id: int,
         supercategory: str,
         name: str,
         keypoints: list[str],
@@ -192,7 +155,7 @@ class Category(BaseField):
         """Keypoint Detection Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
             keypoints (list[str]): category name.
@@ -202,24 +165,24 @@ class Category(BaseField):
             Category: category class
         """
         return cls(
-            cat_id, supercategory, name, keypoints=keypoints, skeleton=skeleton
+            category_id, supercategory, name, keypoints=keypoints, skeleton=skeleton
         )
 
     @classmethod
     def text_recognition(
-        cls, cat_id: int, supercategory: str, name: str
+        cls, category_id: int, supercategory: str, name: str
     ) -> "Category":
         """Text Recognition Category Format
 
         Args:
-            cat_id (int): category id. natural number.
+            category_id (int): category id. natural number.
             supercategory (str): supercategory name.
             name (str): category name.
 
         Returns:
             Category: category class
         """
-        return cls(cat_id, supercategory, name)
+        return cls(category_id, supercategory, name)
 
     def to_dict(self) -> dict:
         """Get Dictionary of Category
@@ -229,7 +192,7 @@ class Category(BaseField):
         """
 
         cat = {
-            "id": self.cat_id,
+            "category_id": self.category_id,
             "supercategory": self.supercategory,
             "name": self.name,
         }
